"""The 'actions' module provides menus and classes for the main
application window. """
#  MIT Licence
#  Copyright (c) 2024 Asger Jon Vistisen
from __future__ import annotations

from ._abstract_menu import AbstractMenu
from ._files_menu import FilesMenu
from ._edit_menu import EditMenu
from ._help_menu import HelpMenu
from ._debug_menu import DebugMenu
from ._main_status_bar import MainStatusBar
